﻿/*  Copyright (C) 2014 Raphael Freudiger <laser_b@gmx.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Author: Raphael Freudiger <laser_b@gmx.ch>
**/
const Lang = imports.lang;
const Mainloop = imports.mainloop;

const LoginManager = imports.misc.loginManager;
const Main = imports.ui.main;
const StatusSystem = imports.ui.status.system;
const PopupMenu = imports.ui.popupMenu;
const ExtensionSystem = imports.ui.extensionSystem;

const Gettext = imports.gettext.domain('gnome-shell-extension-shutdown-helper');
const _ = Gettext.gettext;

const Me = imports.misc.extensionUtils.getCurrentExtension();
const Lib = Me.imports.lib;

const Util = imports.misc.util;

const Extension = new Lang.Class({
    Name: 'ShutdownHelper.Extension',

    enable: function() {
        this._loginManager = LoginManager.getLoginManager();
        this.systemMenu = Main.panel.statusArea['aggregateMenu']._system;

        this._createActions();
        this._removealtSwitcher();
        this._addSeparateButtons();

        this._menuOpenStateChangedId = this.systemMenu.menu.connect('open-state-changed', Lang.bind(this,
            function(menu, open) {
                if (!open)
                    return;
                this._altrestartAction.visible = true;
                this._altlockScreenAction.visible = true;
                this._altpowerOffAction.visible = true;
            }));
    },

    disable: function() {
        if (this._menuOpenStateChangedId) {
            this.systemMenu.menu.disconnect(this._menuOpenStateChangedId);
            this._menuOpenStateChangedId = 0;
        }

        this._destroyActions();
        this._addDefaultButton();
    },    
    
    _createActions: function() {
        this._altlockScreenAction = this.systemMenu._createActionButton('changes-prevent-symbolic', _("Lock Screen"));
        this._altlockScreenActionID = this._altlockScreenAction.connect('clicked', Lang.bind(this, this._onLockScreenClicked));

        this._altrestartAction = this.systemMenu._createActionButton('view-refresh-symbolic', _("Restart"));
        this._altrestartActionID = this._altrestartAction.connect('clicked', Lang.bind(this, this._onRestartClicked));

        this._altpowerOffAction = this.systemMenu._createActionButton('system-shutdown-symbolic', _("Power Off"));
        this._altpowerOffActionId = this._altpowerOffAction.connect('clicked', Lang.bind(this, this._onPowerOffClicked));
    },
    
    _destroyActions: function() {
        if (this._altlockScreenActionId) {
            this._altlockScreenAction.disconnect(this._altlockScreenActionId);
            this._altlockScreenActionId = 0;
        }

        if (this._altrestartActionId) {
            this._altrestartAction.disconnect(this._altrestartActionId);
            this._altrestartActionId = 0;
        }

        if (this._altpowerOffActionId) {
            this._altpowerOffAction.disconnect(this._altpowerOffActionId);
            this._altpowerOffActionId = 0;
        }
        
        if (this._altlockScreenAction) {
            this._altlockScreenAction.destroy();
            this._altlockScreenAction = 0;
        }

        if (this._altrestartAction) {
            this._altrestartAction.destroy();
            this._altrestartAction = 0;
        }

        if (this._altpowerOffAction) {
            this._altpowerOffAction.destroy();
            this._altpowerOffAction = 0;
        }
    },
    
    _addDefaultButton: function() {
        this.systemMenu._actionsItem.actor.add(this.systemMenu._altSwitcher.actor, { expand: true, x_fill: false });
    },
    
    _addSeparateButtons: function() {
        this.systemMenu._actionsItem.actor.add(this._altlockScreenAction, { expand: true, x_fill: false });
        this.systemMenu._actionsItem.actor.add(this._altrestartAction, { expand: true, x_fill: false });
        this.systemMenu._actionsItem.actor.add(this._altpowerOffAction, { expand: true, x_fill: false });
    },
    
    _removealtSwitcher: function() {
        this.systemMenu._actionsItem.actor.remove_child(this.systemMenu._altSwitcher.actor);
    },
    
    _createaltStatusSwitcher: function() {
        this._altStatusSwitcher = new StatusSystem.AltSwitcher(this._altrestartAction,this._altpowerOffAction);
        this.systemMenu._actionsItem.actor.add(this._altStatusSwitcher.actor, { expand: true, x_fill: false });
    },
    
    _removealtStatusSwitcher: function() {
        if (this._altStatusSwitcher) {
            this.systemMenu._actionsItem.actor.remove_child(this._altStatusSwitcher.actor);
            this._altStatusSwitcher.actor.destroy();
            this._altStatusSwitcher = 0;
        }
    },
    
    _onPowerOffClicked: function() {
        Util.spawn(['sudo', '-n', 'halt'])
    },

    _onRestartClicked: function() {
        Util.spawn(['sudo', '-n', 'reboot'])
    },

    _onLockScreenClicked: function() {
        Util.spawn(['gnome-screensaver-command', '--lock'])
    }
});

function init(metadata) {
    Lib.initTranslations(Me);
    return (extension = new Extension());
}

